/*
Initialize spins and couplings.
*/

#include "comext.h"
#include <stdio.h>
#include <math.h>

char backname[128] ;
char jfilename[128] ;
char xfilename[128] ;

float dilution=0.0, actual_dilution=0.0;

void init(spin,flag)     /* initialize spins */
short spin[IMAX][JMAX];
int flag;
{ 
   int i,j;
   FILE *fd;
   double pranf();

   switch(flag) {

     case 0 :  /* COLD (ordered) start */
       	       printf("Cold start, totswps=0\n");
	       totswps=0;
	       for (i=0;i<isiz;i++) {
	       for (j=0;j<jsiz;j++) {
                  spin[i][j] = 0; 
	       }}
	       break;

     case 1 :  /* HOT (random) start */
	       printf("Hot start, totswps=0\n");
	       totswps=0;
	       for (i=0;i<isiz;i++) {
	       for (j=0;j<jsiz;j++) {
	          spin[i][j] = pranf() * Q;
	          if (spin[i][j] == Q) spin[i][j] = 0;
               }}
	       break;

     case 2 :  /* RESTART (from a file) */
	       sprintf(backname, "back.%s", lastfname);
	       if((fd=fopen(backname,"r")) == NULL) {
		 perror("open") ;
	 	 exit(-1) ;
	       }
	       getdata(fd,spin) ;
	       fclose(fd) ;
	       break;

     default : /* ERROR */
	       perror("initialisation");
	       exit(1);

   } /* end switch */
}


void jinit(flag)     /* initialize couplings */
int flag;
{ 
int i,j,d, bound;
double coupling,abs_coupling;
double pranf(),normal();

   switch(flag) {

     case 0 :  /* constant ferromagnetic */
       	       printf("Constant ferromagnetic coupling\n");
	       ferromag = TRUE;
	       break;

     case 1 :  /* constant anti-ferromagnet */
       	       printf("Constant anti-ferromagnetic coupling\n");
	       ferromag = FALSE;
	       beta *= -1.0;
	       break;

     case 2 :  /* Read J[i][j] from a file */
	       readJ();
	       break;

     case 3 :  /* random ferromagnetic */
       	       printf("Random ferromagnetic couplings\n");
	       ferromag = TRUE;
               /* Set up the couplings */
	       for (i=0;i<isiz;i++) {
	       for (j=0;j<jsiz;j++) {
	          for (d=0;d<DIMENSION;d++) {
                     J[i][j][d] = pranf(); 
	          }
	       }}
	       break;

     case 4 :  /* random anti-ferromagnet */
       	       printf("Random anti-ferromagnetic couplings\n");
	       ferromag = FALSE;
               /* Set up the couplings */
	       for (i=0;i<isiz;i++) {
	       for (j=0;j<jsiz;j++) {
	          for (d=0;d<DIMENSION;d++) {
                     J[i][j][d] = - pranf(); 
	          }
	       }}
	       break;

     case 5 :  /* +/-J spin glass */
       	       printf("+/-J spin glass\n");
	       ferromag = FALSE;
	       glass = TRUE;
               frustrated = TRUE;
               /* Set up the couplings */
	       if (first == 1) { 
	          actual_dilution = 0.0;
	          for (i=0;i<isiz;i++) {
	          for (j=0;j<jsiz;j++) {
	             for (d=0;d<DIMENSION;d++) {
                        J[i][j][d] = 1.0; 
			if (pranf() < 0.5) 
                           J[i][j][d] *= -1.0; 
			actual_dilution += J[i][j][d]; 
	             }
	          }}
	       }
	       else {
		  readJ();
	          actual_dilution = 0.0;
	          for (i=0;i<isiz;i++) {
	          for (j=0;j<jsiz;j++) {
	             for (d=0;d<DIMENSION;d++) {
			actual_dilution += J[i][j][d]; 
		     }
	          }}
	       }
	       printf("Actual dilution = %f\n", actual_dilution);
	       break;

     case 6 :  /* Gaussian spin glass */
       	       printf("Gaussian spin glass\n");
	       ferromag = FALSE;
	       glass = TRUE;
               frustrated = TRUE;
               /* Set up the couplings */
	       for (i=0;i<isiz;i++) {
	          for (j=0;j<jsiz;j++) {
	             for (d=0;d<DIMENSION;d++) {
                        coupling = normal();
                        J[i][j][d] = coupling; 
	             }
	          }
	       }
	       break;

     case 7 :  /* Fully frustrated Ising model - All horizontal bonds are 
		* ferro, while vertical bonds are ferro for even columns 
		* and anti-ferro for odd columns
		*/
       	       printf("Fully frustrated Ising model\n");
               ferromag = FALSE;
               frustrated = TRUE;
               /* Set up the couplings */
	       for (i=0;i<isiz;i++) {
	       for (j=0;j<jsiz;j++) {
	          for (d=0;d<DIMENSION;d++) {
                     J[i][j][d] = 1.0; 
	          }
                  if ((i % 2) == 1) J[i][j][1] *= -1.0; 
	       }}
	       break;

     case 8 :  /* Dilute fully frustrated Ising model */
       	       printf("Dilute fully frustrated Ising model\n");
	       printf("Dilution? ");
	       scanf("%f", &dilution);
	       printf("%f\n", dilution);
               ferromag = FALSE;
               frustrated = TRUE;
               glass = TRUE;
               /* Set up the couplings */
	       if (first == 1) { 
	          actual_dilution = 0.0;
	          for (i=0;i<isiz;i++) {
	          for (j=0;j<jsiz;j++) {
	             for (d=0;d<DIMENSION;d++) {
                        J[i][j][d] = 1.0; 
			if (pranf() < dilution) {
			   J[i][j][d] = 0.0;
			   actual_dilution += 1.0;
			}
	             }
                     if ((i % 2) == 1) J[i][j][1] *= -1.0; 
	          }}
	          actual_dilution /= (float)(sites * DIMENSION);
	       }
	       else {
		  readJ();
	          actual_dilution = 0.0;
	          for (i=0;i<isiz;i++) {
	          for (j=0;j<jsiz;j++) {
	             for (d=0;d<DIMENSION;d++) {
			if (J[i][j][d] == 0.0) 
			   actual_dilution += 1.0;
		     }
	          }}
	          actual_dilution /= (float)(sites * DIMENSION);
	       }
	       printf("Actual dilution = %f\n", actual_dilution);
	       break;

     default : /* ERROR */
	       perror("initialisation of couplings");
	       exit(1);

   } /* end switch */

   /* Set up the probabilities */
   for (i=0;i<isiz;i++) {
   for (j=0;j<jsiz;j++) {
      for (d=0;d<DIMENSION;d++) {
	 coupling = beta * J[i][j][d];
	 if (coupling < 0.0) 
	    abs_coupling = -coupling;
	 else 
	    abs_coupling = coupling;
         P[i][j][d] = 1 - exp(-(double)(abs_coupling * J[i][j][d])); 
	 if ((beta == 0.0) && (J[i][j][d] != 0))  /* Zero temperature */
            P[i][j][d] = 1;
      }
   }}

   /* Print out the couplings */
   if (glass) writeJ();

}



print_coupling(istart,jstart,iend,jend)
{
int i,j;

   for (i=istart;i<iend;i++) {
      for (j=jstart;j<jend;j++) {
	 printf("%f  ", J[i][j][0]);
      }
      printf("\n");
   }
   printf("\n");

   return 0;
}


readJ()
{
int i,j,d,two_jsiz,ret;
FILE *fd;

   sprintf(jfilename, "J.%s", lastfname) ;
   if((fd=fopen(jfilename,"r")) == NULL) {
	 perror("J file open") ; exit(-1) ;
   }
   printf("Reading couplings from file %s\n", jfilename); 

   two_jsiz = DIMENSION * jsiz;
   for (i=0;i<isiz;i++) {
      ret=fread(&J[i][0][0],sizeof(float),two_jsiz,fd);
      if (ret != two_jsiz) {
         perror("J file read") ; exit(-2) ;
      }
   }
   fclose(fd);
}

writeJ()
{
int i,j,d,two_jsiz,ret;
FILE *fd;

   sprintf(jfilename, "J.%s", fname) ;
   if((fd=fopen(jfilename,"w")) == NULL) {
	 perror("J file open") ; exit(-2) ;
   }
   two_jsiz = DIMENSION * jsiz;
   for (i=0;i<isiz;i++) {
      cfwrite(&J[i][0][0],sizeof(float),two_jsiz,fd);
   }
   fclose(fd);
}


